clear all;
close all;
getd = @(p)path(p,path);
MyPath='data/eg1/';
getd('solvers/')
%% Set Parameters
L           = 2;
ParamsTV.L=L;


%%  Read Image

% Below X is n1 x n2 x 3; i.e. we have three n1 x n2 images, one for each of the 3 color channels
namestr = 'Cameraman' ;stanstr = 'tif'      ;

X       = mat2gray(imread([namestr,'.',stanstr]));

%eta=.1;ParamsTV.r1=2;ParamsTV.r2=2e4;ParamsTV.lambda=100;
eta=.01;ParamsTV.r1=5;ParamsTV.r2=5e4;ParamsTV.lambda=2e2;

ParamsTV.itmax=50;
ParamsTV.flag=1;
X=eta*X;


Params=ParamsTV;


n1      = size(X,1)                               ;
n2      = size(X,2)                               ;

%% Make masks and linear sampling operators

randn('state',2014);
rand ('state',2014);

L = Params.L;                  % Number of masks
Masks = zeros(n1,n2,L);  % Storage for L masks, each of dim n1 x n2
m = n1* n2* L;

% Sample phases: each symbol in alphabet {1, -1, i , -i} has equal prob.
s = RandStream('mt19937ar','Seed',0);RandStream.setGlobalStream(s);

for ll = 1:L, Masks(:,:,ll) = randsrc(n1,n2,[1i -1i 1 -1]); end
temp = rand(size(Masks));
Masks = Masks .* ( (temp <= 0.2)*sqrt(3) + (temp > 0.2)/sqrt(2) );


% Make linear operators;
A = @(I)  fft2(conj(Masks) .* reshape(repmat(I,[1 L]), size(I,1), size(I,2), L));  % Input is n1 x n2 image, output is n1 x n2 x L array
At= @(Y) sum(Masks .* ifft2(Y), 3) * size(Y,1) * size(Y,2);
AtA=sum(Masks .*conj(Masks), 3) * n1*n2;


%%

x = squeeze(X(:,:,1)); % Image x is n1 x n2
Y = abs(A(x)).^2;        % Measured data
YPoiss= poissrnd(Y);

Params.n1=n1;Params.n2=n2;


disp(['Starting to compute PR without TV!'])
tic;
[u,uInit] = solverPRPoisson(YPoiss,x,  A, At,Params,AtA);
toc;
disp(['Finshing computing PR without TV and Starting to computing PR with TV!'])
tic;
[uTV,uTVInit] = solverTVPRPoisson(YPoiss,x,  A, At,ParamsTV,AtA);

toc;
%%

SNR  =snrCompt(u,x);
SNRTV=snrCompt(uTV,x);

if ~exist(MyPath,'dir')
    mkdir(MyPath);
end

eta=eta*100;

figure;
subplot(121);imshow(abs(u),[]);title(['SNR=',num2str(SNR)]);
subplot(122);imshow(abs(uTV),[min(x(:)),max(x(:))]);title(['+TV,SNR=',num2str(SNRTV)]);
print([MyPath,num2str(namestr),num2str(eta,2),'.png'],'-dpng');
