function [u,uInit,errG]=solverTVPRPoissonRevise(Y,x,A,At,Params,AtA,Omega)
%profile on
if nargin<7
    Omega=ones(size(x));
end

verbose=0;

[r,c,L]=size(Y);
n = r*c;
Omega= reshape(repmat(Omega,[1 L]), r, c, L);
Y=Y.*Omega;

r1=Params.r1;
r2=Params.r2;
lambda=Params.lambda;
itmax=Params.itmax;
%Build A matrix
C=spdiags(ones(n,1),0,n,n);
C=4*C-circshift(C,[0,-1])-circshift(C,[0,1])-circshift(C,[0,r])-circshift(C,[0,-r]);
C=sparse(C);


%B=@(x) (reshape(real(At(A(x)))/r1,1,r*c)+C*x(:)/r2);
%B=@(x) (reshape(real(AtA(reshape(x,r,c))),n,1)/r1+C*x/r2);
B=spdiags(real(AtA(:)),0,n,n)*r1+C*r2;


%spectral initialization
%{
npower_iter = 50;                          % Number of power iterations
z0 = randn(r,c); z0 = z0/norm(z0,'fro'); % Initial guess
for tt = 1:npower_iter,
    z0 = At(Y.*A(z0)); z0 = z0/norm(z0,'fro');
end
normest = sqrt(sum(Y(:))/numel(Y)); % Estimate norm to scale eigenvector
z = normest * z0;                   % Apply scaling
u0=abs(z);
uInit=u0;
%}


%%

if ~isfield('Params','init')
    z0 = randn(r,c); z0 = z0/norm(z0,'fro'); % Initial guess
    normest = sqrt(sum(Y(:))/numel(Y)); % Estimate norm to scale eigenvector
    z = normest * z0;                   % Apply scaling
    u0=abs(z);
    %
else
    u0=Params.init;
end

uInit=u0;
%%
z0= A(u0);
p0= grad2(u0);
q0=zeros(r,c*2);
v0=zeros(r,c,L);

step=1;

for ii=1:itmax
    [~,scale]=snrComptC(u0,x);errG(ii)=norm(scale*u0-x,'fro');
    if isreal(x)
        g=real(At((z0+v0/r1)))*r1-diver2(p0+q0/r2)*r2;
    else
        g=(At((z0+v0/r1)))*r1-diver2(p0+q0/r2)*r2;
    end
    if ii==1
        [uu,~]= cgs(B,g(:));
    else
        [uu,~]= cgs(B,g(:),1e-5,2,[],[],u0(:));
    end
    u=reshape(uu,r,c);
    
    %step II for z
    tmp=A(u)-v0/r1;
    z=(sign(tmp).*(r1*abs(tmp)+sqrt(r1^2*abs(tmp).^2+4*(1+r1)*Y))/2/(1+r1)).*Omega+(1-Omega).*tmp;
    %step III for p
    p=softThresh(grad2(u)-q0/r2,lambda/r2);
    
    %Step IV; updat for multipliers
    
    
    
    
    v=v0+r1*step*(z-A(u));
    q=q0+r2*step*(p-grad2(u));
    
    %% update the variables
    v0=v;q0=q;
    z0=z;p0=p;
    u0=u;
    
    %
    if verbose==1
        figure(1);imshow(u,[]);
        title([num2str(ii),'_{th} SNR=',num2str(snrCompt(u,x))]);
        drawnow;
    end
    
    
    
end


end